/*
 * @(#)ComponentTest.java	1.31 95/09/01 Sami Shaio
 *
 * Copyright (c) 1995 Sun Microsystems, Inc. All Rights Reserved.
 *
 * Permission to use, copy, modify, and distribute this software
 * and its documentation for NON-COMMERCIAL purposes and without
 * fee is hereby granted provided that this copyright notice
 * appears in all copies. Please refer to the file "copyright.html"
 * for further important copyright and licensing information.
 *
 * SUN MAKES NO REPRESENTATIONS OR WARRANTIES ABOUT THE SUITABILITY OF
 * THE SOFTWARE, EITHER EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
 * TO THE IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
 * PARTICULAR PURPOSE, OR NON-INFRINGEMENT. SUN SHALL NOT BE LIABLE FOR
 * ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR
 * DISTRIBUTING THIS SOFTWARE OR ITS DERIVATIVES.
 */

import java.awt.*;
import java.applet.*;
import java.util.Date;

public class ComponentTest extends Applet
{
    Panel 	center;

    public void init()
    {
	setBackground(Color.lightGray);
	setFont(new Font("Helvetica", Font.PLAIN, 12));
	setLayout(new BorderLayout());
	Label l  = new Label("North");
	l.setAlignment(Label.CENTER);

	add("South", new TextField("South"));
	add("North", l);
	center = new ScrollPanel();
	center.setLayout(new BorderLayout());
	add("Center", center);

	Panel innerPanel = new ScrollPanel();
	center.add("Center", innerPanel);
	innerPanel.setLayout(new FlowLayout(FlowLayout.CENTER, 5, 5));
	innerPanel.add(new Label("List of Widgets"));
	innerPanel.add(new Button("Arthur"));
	innerPanel.add(new Button("Sami"));
	innerPanel.add(new SillyWidget());
	innerPanel.add(new TextField("quite random"));
	innerPanel.add(new Scrollbar(Scrollbar.VERTICAL));
	innerPanel.add(new Scrollbar(Scrollbar.HORIZONTAL));

	List li = new List(4, false);

	li.reshape(0, 0, 100, 100);
	li.addItem("Arthur");
	li.addItem("Sami");
	li.addItem("James");
	li.addItem("Chris");
	li.addItem("Mary");
	li.addItem("Lisa");
	li.addItem("Kathy");
	li.select(1);

	innerPanel.add(li);
	TextArea tx = new TextArea(10, 5);
	tx.setText("this is some random text");
	tx.setForeground(Color.red);

	innerPanel.add(tx);

	Choice	opt = new Choice();

	opt.addItem("Arthur");
	opt.addItem("Sami");
	opt.addItem("Chris");
	opt.addItem("Jim");

	innerPanel.add(opt);

	CheckboxGroup g = new CheckboxGroup();
	innerPanel.add(new Checkbox("one", g, false));
	innerPanel.add(new Checkbox("two", g, true));
	innerPanel.add(new Checkbox("three", g, false));
	center.add("East", new Scrollbar(Scrollbar.VERTICAL));
	center.add("South", new Scrollbar(Scrollbar.HORIZONTAL));

	add("East", new Button("East"));
	add("West", new Button("West"));
    }
	    
        public synchronized boolean handleEvent(Event e)
        {
		return false;
	}
}


class ScrollPanel extends Panel {
    public ScrollPanel() {
	reshape(0, 0, 100, 100);
    }

    public void paint(Graphics g) {
	Rectangle r = bounds();

	g.drawRect(0, 0, r.width, r.height);
    }
}

class SillyWidget extends Canvas {
    Color	c1 = Color.pink;
    Color	c2 = Color.lightGray;
    long	entered = 0;
    long	inComponent = 0;
    Font	font = new Font("Courier", Font.BOLD, 12);
    FontMetrics fm;

    public SillyWidget() {
	reshape(0, 0, 100, 100);
    }

    public boolean mouseEnter(Event e, int x, int y) {
	Color c = c1;
	entered = e.when;
	c1 = c2;
	c2 = c;
	repaint();
	return true;
    }

    public boolean mouseExit(Event e, int x, int y) {
	Color c = c1;

	inComponent = e.when - entered;
	c1 = c2;
	c2 = c;
	repaint();
	return true;
    }

    public void paint(Graphics g) {
	Rectangle	r = bounds();

	g.setColor(c1);
	g.fillRoundRect(0, 0, r.width, r.height, 20, 20);
	g.setColor(c2);
	g.fillRoundRect(5, 5, r.width - 10, r.height - 10, 20, 20);
	g.setColor(c1);
	g.fillRoundRect(15, 15, r.width - 30, r.height - 30, 20, 20);
	g.setColor(Color.darkGray);
	g.setFont(font);
	fm = g.getFontMetrics(font);
	String s = ""+inComponent+" ms.";
	int width = fm.stringWidth(s);
	System.out.println("width = " + width);
	g.drawString(s,
		     (r.width - width) / 2,
		     (r.height - fm.getHeight()) / 2);
    }
}
