// frog.java -- jumping frog animation applets
//
// author :  Sandra Loosemore
// date   :  13 Sep 1995
//


import browser.*;
import browser.audio.*;
import awt.*;



//=======================================================================
// Generic leap frog applet
//=======================================================================


class FrogApplet extends Applet {

  // icons and sounds
  static Image images[][] = new Image[3][4];
  static AudioData croak;

  // constants
  static final int IMAGESIZE = 32;
  static final int WIDTH = 500;
  static final int HEIGHT = 300;

  // frogs and threads
  int nfrogs;
  Frog frogs[];
  HopThread thread = null;
  boolean allsquashed = false;

  // for double-buffering
  Image ibuffer;
  Graphics gbuffer;

  // initialization on applet startup
  public void init ()  {
    resize(WIDTH, HEIGHT);
    images[Frog.SITTING][0] = getImage("sitting0.gif");
    images[Frog.HOPPING][0] = getImage("hopping0.gif");
    images[Frog.SQUASHED][0] = getImage("squashed0.gif");
    images[Frog.SITTING][1] = getImage("sitting1.gif");
    images[Frog.HOPPING][1] = getImage("hopping1.gif");
    images[Frog.SQUASHED][1] = getImage("squashed1.gif");
    images[Frog.SITTING][2] = getImage("sitting2.gif");
    images[Frog.HOPPING][2] = getImage("hopping2.gif");
    images[Frog.SQUASHED][2] = getImage("squashed2.gif");
    images[Frog.SITTING][3] = getImage("sitting3.gif");
    images[Frog.HOPPING][3] = getImage("hopping3.gif");
    images[Frog.SQUASHED][3] = getImage("squashed3.gif");
    croak = getAudioData("croak.au");
    frogs = new Frog[nfrogs];
    for (int i = 0; i<nfrogs; i++) {
      Frog f = new Frog();
      frogs[i] = f;
      f.x = xrandom ();
      f.y = yrandom ();
      f.direction = irandom (4);
      f.targetx = xrandom ();
      f.targety = yrandom ();
      }
    try {
      ibuffer = createImage(WIDTH, HEIGHT);
      gbuffer = new Graphics(ibuffer);
      paintApplet (gbuffer);
      }
    catch (Exception e)
      gbuffer = null;
    }

  // random number utilities
  int irandom (int max) {
    return (int) (Math.random() * max);
    }

  int xrandom () {
    return irandom(WIDTH - IMAGESIZE);
    }

  int yrandom () {
    return irandom(HEIGHT - IMAGESIZE);
    }


  // Graphics methods.
  // If the browser supports doublebuffering, use it to avoid flickering.  
  // Do all the drawing of the frogs to the buffer, then just zap the 
  // buffer image onto the screen.
  public void paint (Graphics g) {
    update (g);
    }

  public void update (Graphics g) {
    if (gbuffer == null)
      // no double-buffering; draw to screen.
      paintApplet (g);
    else  {
      // draw from buffer
      g.drawImage (ibuffer, 0, 0);
      }
    }

  void paintApplet (Graphics g) {
    // for some reason clearRect doesn't clear the background when
    // this is used with double-buffering.....
    g.SetColor (Color.lightGray);
    g.paint3DRect (0, 0, WIDTH, HEIGHT, true, true);
    for (int i = 0; i < nfrogs; i++) {
      Frog f = frogs[i];
      g.drawImage (images[f.status][f.direction], f.x, f.y);
      }
    }

  void redraw () {
    if (gbuffer != null)  paintApplet (gbuffer);
    repaint ();
    }

  // start/stop animation thread when mouse moves in and out of window.
  public void mouseEnter () {
    if (thread == null) {
      thread = new HopThread (this);
      thread.start ();
      }
    else
      thread.resume();
    }

  public void mouseExit () {
    thread.suspend();
    }

  // clean up on exit from page.
  public void stop () {
    if (thread != null)
      thread.stop ();
    }

  public void destroy () {
    if (gbuffer != null) {
      gbuffer.dispose ();
      ibuffer.dispose ();
      }
    }


  // this method is specialized in subclasses.
  void getNewTarget (Frog f) {
    }

  }  // end of class FrogApplet



//=======================================================================
// subclass of applets with one frog that hops towards cursor
//=======================================================================

class FrogApplet1 extends FrogApplet {

  public void init () {
    nfrogs = 1;
    super.init ();
    }

  // mouse movement method; give frog new target and wake it up.
  public void mouseMove (int newx, int newy) {
    frogs[0].targetx = newx - IMAGESIZE/2;
    frogs[0].targety = newy - IMAGESIZE/2;
    thread.resume ();
    } // end of mouseMove

  // at target notification.  croak and put it to sleep.
  void getNewTarget (Frog f) {
    thread.suspend ();
    play (croak);
    }
  }  // end of class FrogApplet1



//=======================================================================
// Applet subclass with multiple frogs that hop around randomly.
//=======================================================================

class FrogApplet2 extends FrogApplet {

  static final int NFROGS = 10;

  public void init () {
    nfrogs = NFROGS;
    super.init ();
    }

  // at target notification.  give it a new target.
  void getNewTarget (Frog f) {
    f.targetx = xrandom();
    f.targety = yrandom();
  }


  // mouse click when all squashed reinitializes; otherwise squashes frog
  // under cursor.
  public void mouseUp (int x, int y) {
    if (allsquashed) {
      for (int i = 0; i<nfrogs; i++)
        frogs[i].status = Frog.SITTING;
      allsquashed = false;
      thread.resume ();
      }
    else
      for (int i = 0; i<nfrogs; i++)  {
        Frog f = frogs[i];
        if (f.status != Frog.SQUASHED)  {
          int dx = x - IMAGESIZE/2 - f.x;
          int dy = y - IMAGESIZE/2 - f.y;
          int dxabs = Math.abs(dx);
          int dyabs = Math.abs(dy);
          if ((dxabs <= IMAGESIZE) && (dyabs <= IMAGESIZE))  {
            play (croak);
            f.status = Frog.SQUASHED;
            redraw ();
            return;
            }
          }
        }
    }

  }  // end of class FrogApplet2



//=======================================================================
// Class of frog objects.
//=======================================================================

class Frog extends Object {

  // location and orientation of frog
  int x = 0;
  int y = 0;
  int direction = 0;
  int status = SITTING;

  // constants
  static final int SITTING = 0;
  static final int HOPPING = 1;
  static final int SQUASHED = 2;
  static final int HOPSIZE = HopThread.HOPSTEP * HopThread.HOPINC;

  // target location
  int targetx = 0;
  int targety = 0;

 // set new direction for frog to hop towards target.
 // if frog is at target, request new target from applet.
  void setDirection (FrogApplet a) {
    int dx = targetx - x;
    int dy = targety - y;
    int dxabs = Math.abs(dx);
    int dyabs = Math.abs(dy);

    if ((dxabs <= HOPSIZE) && (dyabs <= HOPSIZE))  {
      a.getNewTarget (this);
      setDirection (a);
      }
    else if (dy >= dxabs)
      direction = 0;
    else if ((- dx) >= dyabs)
      direction = 1;
    else if ((- dy) >= dxabs)
      direction = 2;
    else
      direction = 3;
    }
  } // end of class Frog



//=======================================================================
// Hop animation thread class
//=======================================================================

class HopThread extends Thread {

  FrogApplet applet;

  // constants
  static final int HOPSTEP = 4;
  static final int HOPINC = 8;
  static final int HOPTIME = 50;
  static final int SITTIME = 200;

  // hop increments by direction
  static final int hopdx[] = new int[4];
  static final int hopdy[] = new int[4];

  static {
    hopdx[0] = 0; hopdy[0] = HOPINC;
    hopdx[1] = - HOPINC; hopdy[1] = 0;
    hopdx[2] = 0; hopdy[2] = - HOPINC;
    hopdx[3] = HOPINC; hopdy[3] = 0;
    }

  HopThread (FrogApplet a) {
    applet = a;
    }

  // hop all the frogs in unison.

  public void run () {
    int nfrogs = applet.nfrogs;
    while (true)  {

      // Compute the direction for each frog to hop in.  
      // If all frogs are squashed, suspend the animation thread since
      // there is nothing to hop.
      boolean allsquashed = true;
      while (allsquashed) {
        for (int i=0; i<nfrogs; i++) {
          Frog f = applet.frogs[i];
          if (f.status != Frog.SQUASHED)  {
            f.setDirection(applet);
            allsquashed = false;
            }
          }
        if (allsquashed) {
          applet.allsquashed = true;
          suspend();
          }
        }

      // Do the animation.
      applet.redraw ();
      for (int step = 1; step <= HOPSTEP; step++) {
        sleep (HOPTIME);
        for (int i=0; i<nfrogs; i++) {
          Frog f = applet.frogs[i];
          if (f.status != Frog.SQUASHED) {
            f.x = f.x + hopdx[f.direction];
            f.y = f.y + hopdy[f.direction];
            if (step < HOPSTEP)
              f.status = Frog.HOPPING;
            else
              f.status = Frog.SITTING;
            }
          }
        applet.redraw ();
        }
      sleep (SITTIME);
      }
    } // end of run method


  }  // end of class HopThread


